/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef lint
static char sccsid[] = "@(#)$RCSfile: lcg-cr.c,v $ $Revision: 1.25 $ $Date: 2009/11/18 13:53:33 $ CERN Jean-Philippe Baud";
#endif /* not lint */

/*	lcg-cr - copy a file to a Storage Element and registers it in the LRC
 *	with a specified GUID
 */
#include <sys/types.h>
#include <errno.h>
#include <getopt.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "lcg_util.h"


main (int argc, char **argv)
{
	int helpflg = 0;
	int insecure = 0;
	int verbose = 0;
	int version = 0;
	char actual_guid[37];
	int c;
	char *conf_file = NULL;
	char *dest_file = NULL;
	char *dp;
	int errflg = 0;
	int timeout = 0, timeout_tmp = 0;
	char *guid = NULL;
	char *lfn_arg = NULL;
	char** dst_protocols = NULL;
	static struct option longopts[] = {
		{"config", required_argument, NULL, 'c'},
		{"help", no_argument, NULL, 'h'},
		{"insecure", no_argument, NULL, 'i'},
		{"st", required_argument, NULL, 's'},
		{"defaultsetype", required_argument, NULL, 'D'},
		{"setype", required_argument, NULL, 'T'},
		{"nobdii", no_argument, NULL, 'b'},
		{"timeout", required_argument, NULL, 't'},
#define CONNECT_TIMEOUT_ARG_ID 130
		{"connect-timeout", required_argument, NULL, CONNECT_TIMEOUT_ARG_ID},
#define SENDRECEIVE_TIMEOUT_ARG_ID 131
		{"sendreceive-timeout", required_argument, NULL, SENDRECEIVE_TIMEOUT_ARG_ID},
#define BDII_TIMEOUT_ARG_ID 132
		{"bdii-timeout", required_argument, NULL, BDII_TIMEOUT_ARG_ID},
#define SRM_TIMEOUT_ARG_ID 133
		{"srm-timeout", required_argument, NULL, SRM_TIMEOUT_ARG_ID},
		{"verbose", no_argument, NULL, 'v'},
		{"checksum", no_argument, NULL, 'k'},
		{"checksum-type", required_argument, NULL, 'K'},
		{"vo", required_argument, NULL, 'V'},
#define DST_PROTOCOLS_ARG_ID 134
        {"dst-protocols", required_argument, NULL, DST_PROTOCOLS_ARG_ID},
		{"version", no_argument, NULL, 'W'},
		{"nofilesizecheck", no_argument, NULL, 'f'},
		{0, 0, 0, 0}
	};
	int nbstreams = 1;
	char *relative_path = NULL;
	char *spacetokendesc = NULL;
	char *vo = NULL;
	enum se_type defaulttype = TYPE_NONE, setype = TYPE_NONE;
	enum gfal_cksm_type cksmtype = GFAL_CKSM_NONE;
	int nobdii = 0;
	int nofilsizecheck = 0;

	opterr = 1;

	while ((c = getopt_long (argc, argv, "c:bd:D:T:g:hikK:l:n:P:vt:s:W:f", longopts, NULL)) != EOF) {
		switch (c) {
			case 'c':
				conf_file = optarg;
				break;
			case 'd':
				dest_file = optarg;
				break;
			case 'g':
				guid = optarg;
				break;
			case 'h':
				helpflg = 1;
				break;
			case 'i':
				insecure = 1;
				break;
			case 'l':
				lfn_arg = optarg;
				break;
			case 'n':
				if ((nbstreams = strtol (optarg, &dp, 10)) < 1 ||
						*dp != '\0') {
					fprintf (stderr,
							"invalid number of streams %s\n", optarg);
					errflg++;
				}
				break;
			case 'P':
				relative_path = optarg;
				break;
			case 's':
				spacetokendesc = optarg;
				break;
			case 'D':
				if (!strcasecmp (optarg, "srmv1"))
					defaulttype = TYPE_SRM;
				else if (!strcasecmp (optarg, "srmv2"))
					defaulttype = TYPE_SRMv2;
				else if (!strcasecmp (optarg, "se"))
					defaulttype = TYPE_SE;
				else {
					fprintf (stderr,"Invalid SE type: %s\n", optarg);
				}
				break;
			case 'T':
				if (!strcmp (optarg, "srmv1"))
					setype = TYPE_SRM;
				else if (!strcmp (optarg, "srmv2"))
					setype = TYPE_SRMv2;
				else if (!strcmp (optarg, "se"))
					setype = TYPE_SE;
				else {
					fprintf (stderr,"Invalid SE type: %s\n", optarg);
					errflg++;
				}
				break;
			case 'b':
				nobdii = 1;
				break;
			case 'V':
				vo = optarg;
				break;
			case 'v':
				++verbose;
				break;
			case 'k':
				if (cksmtype == GFAL_CKSM_NONE)
					cksmtype = GFAL_CKSM_DEFAULT;
				break;
			case 'K':
				cksmtype = gfal_parse_checksum_algname (optarg);
				if (cksmtype == GFAL_CKSM_NONE) {
					fprintf (stderr,"Invalid checksum type: %s\n", optarg);
				}
				break;
			case 't':
				fprintf (stderr, "Warning: -t,--timeout is deprecated! Use --timeout-* options instead\n");
				if ((timeout = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--timeout': %s\n", optarg);
					errflg++;
				}
			case CONNECT_TIMEOUT_ARG_ID: // "connect-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--connect-timeout': %s\n", optarg);
					errflg++;
				} else
					gfal_set_timeout_connect (timeout_tmp);
				break;
			case SENDRECEIVE_TIMEOUT_ARG_ID: // "sendreceive-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--sendreceive-timeout': %s\n", optarg);
					errflg++;
				} else
					gfal_set_timeout_sendreceive (timeout_tmp);
				break;
			case BDII_TIMEOUT_ARG_ID: // "bdii-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--bdii-timeout': %s\n", optarg);
					errflg++;
				} else
					gfal_set_timeout_bdii (timeout_tmp);
				break;
			case SRM_TIMEOUT_ARG_ID: // "srm-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--srm-timeout': %s\n", optarg);
					errflg++;
				} else
					gfal_set_timeout_srm (timeout_tmp);
				break;
            case DST_PROTOCOLS_ARG_ID:
                dst_protocols = lcg_tokenize(optarg, PROTOCOL_LIST_DELIMITER);

                if (!dst_protocols) {
                    fprintf (stderr, "invalid destination protocol list format: %s\n", optarg);
                    errflg++;
                }
                break;
			case 'W':
				version = 1;
				break;
			case 'f':
				nofilsizecheck = 1;
				break;
			case '?':
				errflg++;
				break;
			default:
				break;
		}
	}
	if (version) {
		printf ("lcg_util-%s\n", lcg_util_version ());
		printf ("GFAL-client-%s\n", gfal_version ());
		exit (0);
	}
	if (optind + 1 != argc) {
		errflg++;
	}
	if (errflg || helpflg) {
		fprintf (stderr, "usage: %s %s%s%s%s%s", argv[0],
				"[-c,--config config_file] [-d dest_file | dest_host] [-g guid] [--checksum] [--checksum-type type]\n",
				"\t[-D,--defaultsetype se|srmv1|srmv2] [-T,--setype se|srmv1|srmv2] [-b,--nobdii]\n",
				"\t[-h,--help] [-l lfn] [-n nbstreams] [-P relative_path] [-s,--st spacetokendesc]\n",
				"\t[--connect-timeout value] [--sendreceive-timeout value] [--bdii-timeout value]\n",
				"\t[--srm-timeout value] [-v,--verbose] [-V,--vo vo] [-f,--nofilesizecheck] [--version]\n"
				"\t[--dst-protocols <protocol1,protocol2,...>]\n"
				"\tsrc_file\n");
		exit (errflg ? 1 : 0);
	}

	c = lcg_cr6 (argv[optind], dest_file, guid, lfn_arg, defaulttype, setype,
	    nobdii, vo, relative_path, nbstreams, conf_file, insecure, verbose,
	    timeout, spacetokendesc, actual_guid, cksmtype, dst_protocols,nofilsizecheck, NULL, 0);

	lcg_free_string_array(dst_protocols);

	if (*actual_guid)
		printf ("guid:%s\n", actual_guid);
	if (c < 0) {
		if (errno)
			perror ("lcg_cr");
		exit (1);
	}
	exit (0);
}
