/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef lint
static char sccsid[] = "@(#)$RCSfile: lcg-del.c,v $ $Revision: 1.28 $ $Date: 2009/07/07 16:01:34 $ CERN Jean-Philippe Baud";
#endif /* not lint */

/*	lcg-del - deletes one file (either one replica or all replicas).
 *	If it was the last or only valid replica, the entries corresponding to
 *	this GUID are also removed from the RMC.
 */
#include <sys/types.h>
#include <errno.h>
#include <getopt.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "lcg_util.h"


main (int argc, char **argv)
{
	int helpflg = 0;
	int insecure = 0;
	int verbose = 0;
	int version = 0;
	int aflag = 0;
	int force = 0;
	int c;
	char *conf_file = NULL;
	int errflg = 0;
	static struct option longopts[] = {
		{"config", required_argument, NULL, 'c'},
		{"file", required_argument, NULL, 'f'},
		{"dir", no_argument, NULL, 'd'},
		{"help", no_argument, NULL, 'h'},
		{"force", no_argument, NULL, 'F'},
		{"defaultsetype", required_argument, NULL, 'D'},
		{"setype", required_argument, NULL, 'T'},
		{"se", required_argument, NULL, 's'},
		{"nobdii", no_argument, NULL, 'b'},
		{"nolfc", no_argument, NULL, 'l'},
		{"insecure", no_argument, NULL, 'i'},
		{"timeout", required_argument, NULL, 't'},
		{"connect-timeout", required_argument, NULL, 130},
		{"sendreceive-timeout", required_argument, NULL, 131},
		{"bdii-timeout", required_argument, NULL, 132},
		{"srm-timeout", required_argument, NULL, 133},
		{"verbose", no_argument, NULL, 'v'},
		{"vo", required_argument, NULL, 'V'},
		{"version", no_argument, NULL, 'W'},
		{0, 0, 0, 0}
	};
	char *file = NULL;
	char **surls = NULL;
	int nbsurls;
	char *se = NULL;
	char *vo = NULL;
	char *dp;
	int timeout = 0, timeout_tmp = 0;
	enum se_type defaulttype = TYPE_NONE, setype = TYPE_NONE;
	int nobdii = 0;
	int nolfc = 0;
	int rmdir = 0;
	lcg_filestatus *statuses = NULL;
	char errmsg[GFAL_ERRMSG_LEN];
	int i, rc = 0;
    static const int errbufsz = 2048;
    char errbuf[errbufsz];

    errbuf[0] = 0;

	while ((c = getopt_long (argc, argv, "ac:f:dhFD:T:blis:vV:t:W", longopts, NULL)) != EOF) {
		switch (c) {
			case 'a':
				aflag = 1;
				break;
			case 'c':
				conf_file = optarg;
				break;
			case 'f':
				file = optarg;
				break;
			case 'd':
				rmdir = 1;
				break;
			case 'h':
				helpflg = 1;
				break;
			case 'F':
				force = 1;
				break;
			case 'i':
				insecure = 1;
				break;
			case 's':
				se = optarg;
				break;
			case 'D':
				if (!strcasecmp (optarg, "srmv1"))
					defaulttype = TYPE_SRM;
				else if (!strcasecmp (optarg, "srmv2"))
					defaulttype = TYPE_SRMv2;
				else if (!strcasecmp (optarg, "se"))
					defaulttype = TYPE_SE;
				else {
					fprintf (stderr,"Invalid SE type: %s\n", optarg);
				}
				break;
			case 'T':
				if (!strcmp (optarg, "srmv1"))
					setype = TYPE_SRM;
				else if (!strcmp (optarg, "srmv2"))
					setype = TYPE_SRMv2;
				else if (!strcmp (optarg, "se"))
					setype = TYPE_SE;
				else {
					fprintf (stderr,"Invalid SE type: %s\n", optarg);
					errflg++;
				}
				break;
			case 'b':
				nobdii = 1;
				break;
			case 'l':
				nolfc = 1;
				break;
			case 'V':
				vo = optarg;
				break;
			case 'v':
				++verbose;
				break;
			case 't':
				fprintf (stderr, "Warning: -t,--timeout is deprecated! Use --timeout-* options instead\n");
				if ((timeout = strtol (optarg, &dp, 10)) < 0 ||
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--timeout': %s\n", optarg);
					errflg++;
				}
			case 130: // "connect-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 ||
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--connect-timeout': %s\n", optarg);
					errflg++;
				} else
					gfal_set_timeout_connect (timeout_tmp);
				break;
			case 131: // "sendreceive-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 ||
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--sendreceive-timeout': %s\n", optarg);
					errflg++;
				} else
					gfal_set_timeout_sendreceive (timeout_tmp);
				break;
			case 132: // "bdii-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 ||
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--bdii-timeout': %s\n", optarg);
					errflg++;
				} else
					gfal_set_timeout_bdii (timeout_tmp);
				break;
			case 133: // "srm-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 ||
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--srm-timeout': %s\n", optarg);
					errflg++;
				} else
					gfal_set_timeout_srm (timeout_tmp);
				break;
			case 'W':
				version = 1;
				break;
			case '?':
			default:
				errflg++;
				break;
		}
	}
	if (version) {
		printf ("lcg_util-%s\n", lcg_util_version ());
		printf ("GFAL-client-%s\n", gfal_version ());
		exit (0);
	}

	if ((!file && optind >= argc) || (file && optind != argc))
		++errflg;

	if (errflg || helpflg) {
		fprintf (stderr, "usage: %s %s%s%s%s", argv[0],
				"[-a] [-c,--config config_file] [-h,--help] [--force] [-b,--nobdii] [-d,--dir] [--srm-timeout value]\n",
				"\t[--connect-timeout value] [--sendreceive-timeout value] [--bdii-timeout value]\n",
				"\t[-D,--defaultsetype se|srmv1|srmv2] [-T,--setype se|srmv1|srmv2] [-l,--nolfc]\n",
				"\t[-s,--se se] [-v,--verbose] [-V,--vo vo] [--version] (-f file | surl [...])\n");
		exit (errflg ? 1 : 0);
	}
	if (aflag && nolfc) {
		fprintf (stderr, "lcg-del: Options -a and --nolfc are not compatible!");
		exit (1);
	}
	if (rmdir && (aflag || se)) {
		fprintf (stderr, "lcg-del: Options -d,--dir isn't compatible with '-a' and '-s,--se' options!");
		exit (1);
	}

	if (file) {
		if ((nbsurls = import_surls_from_file (file, &surls, NULL, 0)) < 0) {
            gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_ERROR, "%s", errbuf);
			perror ("lcg-del");
			exit (1);
		}
	} else {
		surls = argv + optind;
		nbsurls = argc - optind;
	}

	if (rmdir && nbsurls > 1) {
		fprintf (stderr, "lcg-del: Only ONE directory can be deleted at a time");
		exit (1);
	}

	rc = lcg_del7 (nbsurls, surls, defaulttype, setype, nobdii, nolfc, aflag, se, vo,
			conf_file, insecure, verbose, force, rmdir, timeout, &statuses, errbuf, errbufsz);

	if (statuses == NULL) {
		if (rc == 0) errno = EFAULT;
        gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_ERROR, "%s", errbuf);
		perror ("lcg-del");
		exit (1);
	}

	for (i = 0; i < nbsurls; ++i) {
		if (statuses[i].status != 0) {
			if (statuses[i].explanation) {
				snprintf (errmsg, GFAL_ERRMSG_LEN, "%s", statuses[i].explanation);
				free (statuses[i].explanation);
			} else {
				snprintf (errmsg, GFAL_ERRMSG_LEN, "%s: %s", statuses[i].file, strerror (statuses[i].status));
			}

			gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_ERROR, "%s", errmsg);
            gfal_errmsg (NULL, 0, GFAL_ERRLEVEL_ERROR, "%s", errbuf);
		}
		if (statuses[i].file) free (statuses[i].file);
	}

	free (statuses);
	exit (rc == 0 ? 0 : 1);
}
