/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* lcg-ls - list file information for given SURLs or LFNs */
#include <sys/types.h>
#include <errno.h>
#include <getopt.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "gfal_api.h"
#include "lcg_util.h"

#define NBDIR_MAX 512

int display_lfns_results (const char **, int, int, int);
int display_surls_results (gfal_filestatus *, int, int, int, int);


static char *locality_desc[] = {
	"UNKNOWN",
	"ONLINE",
	"NEARLINE",
	"ONLINE_AND_NEARLINE",
	"LOST",
	"NONE",
	"UNAVAILABLE"
};	


static int _lcg_ls_handle_int_option(char* dp, int* errflg, const char* optname)
{
    errno = 0;
    int v = strtol (optarg, &dp, 10);

    if (errno != 0 || 
        *dp != '\0' ) 
    {
        fprintf (stderr, "invalid value for '%s': %s\n", optname, optarg);
        assert(errflg);
        (*errflg)++;
    }

    return v;
}

	int
main(int argc, char **argv)
{
	int helpflg = 0;
	int verbose = 0;
	int version = 0;
	int c;
	int errflg = 0;
	static struct option longopts[] = {
		{"count", required_argument, NULL, 'c'},
		{"offset", required_argument, NULL, 'o'},
		{"long", required_argument, NULL, 'l'},
		{"dir", required_argument, NULL, 'd'},
		{"defaultsetype", required_argument, NULL, 'D'},
		{"setype", required_argument, NULL, 'T'},
		{"nobdii", no_argument, NULL, 'b'},
		{"vo", required_argument, NULL, 'V'},
		{"timeout", required_argument, NULL, 't'},
		{"connect-timeout", required_argument, NULL, 130},
		{"sendreceive-timeout", required_argument, NULL, 131},
		{"bdii-timeout", required_argument, NULL, 132},
		{"srm-timeout", required_argument, NULL, 133},
		{"help", no_argument, NULL, 'h'},
		{"version", no_argument, NULL, 'W'},
		{"verbose", no_argument, NULL, 'v'},
		{0, 0, 0, 0}
	};
	gfal_request req;
	gfal_internal internal;
	gfal_filestatus *filestatuses;
	char *vo = NULL;
	int longlisting = 0;
	int dir = 0;
	int timeout = 0, timeout_tmp = 0;
    int count = 0;
    int offset = 0;
	char *dp;
	int n;

	if ((req = gfal_request_new ()) == NULL) {
		perror ("lcg-ls");
		exit (1);
	}

	while ((c = getopt_long (argc, argv, "c:o:ldD:T:bV:t:hvW", longopts, NULL)) != EOF) {
		switch (c) {
			case 'l':
				longlisting = 1;
				break;
			case 'd':
				dir = 1;
				break;
			case 'D':
				if (!strcasecmp (optarg, "srmv1"))
					req->defaultsetype = TYPE_SRM;
				else if (!strcasecmp (optarg, "srmv2"))
					req->defaultsetype = TYPE_SRMv2;
				else if (!strcasecmp (optarg, "se"))
					req->defaultsetype = TYPE_SE;
				else {
					fprintf (stderr,"Invalid SE type: %s\n", optarg);
				}
				break;
			case 'T':
				if (!strcmp (optarg, "srmv1"))
					req->setype = TYPE_SRM;
				else if (!strcmp (optarg, "srmv2"))
					req->setype = TYPE_SRMv2;
				else if (!strcmp (optarg, "se"))
					req->setype = TYPE_SE;
				else {
					fprintf (stderr,"Invalid SE type: %s\n", optarg);
					errflg++;
				}
				break;
			case 'b':
				req->no_bdii_check = 1;
				break;
			case 'V':
				vo = optarg;
				break;
			case 't':
				fprintf (stderr, "Warning: -t,--timeout is deprecated! Use --timeout-* options instead\n");
				if ((timeout = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--timeout': %s\n", optarg);
					errflg++;
				}
			case 130: // "connect-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--connect-timeout': %s\n", optarg);
					errflg++;
				} else
					gfal_set_timeout_connect (timeout_tmp);
				break;
			case 131: // "sendreceive-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--sendreceive-timeout': %s\n", optarg);
					errflg++;
				} else
					gfal_set_timeout_sendreceive (timeout_tmp);
				break;
			case 132: // "bdii-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--bdii-timeout': %s\n", optarg);
					errflg++;
				} else
					gfal_set_timeout_bdii (timeout_tmp);
				break;
			case 133: // "srm-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--srm-timeout': %s\n", optarg);
					errflg++;
				} else
					gfal_set_timeout_srm (timeout_tmp);
				break;
			case 'W':
				version = 1;
				break;
			case 'h':
				helpflg = 1;
				break;
			case 'v':
				++verbose;
				break;
            /* Handle count option */
			case 'c':
                count = _lcg_ls_handle_int_option(dp, &errflg, "--count");
                break;

            /* Handle offset option */
			case 'o':
                offset = _lcg_ls_handle_int_option(dp, &errflg, "--offset");
                break;

			default:
				fprintf (stderr, "Unknown option %c\n", c);
				errflg++;
				break;
		}
	}

	if (version) {
		printf ("lcg_util-%s\n", lcg_util_version ());
		printf ("GFAL-client-%s\n", gfal_version ());
		exit (0);
	}
	if (optind >= argc)
		errflg++;

	if (errflg || helpflg) {
		fprintf (stderr,
				"usage: %s %s%s%s%s", argv[0],
				"[-l] [-d] [-D,--defaultsetype se|srmv1|srmv2] [-T,--setype se|srmv1|srmv2]\n",
				"\t[--connect-timeout value] [--sendreceive-timeout value] [--bdii-timeout value]\n",
				"\t[--srm-timeout value] [-b,--nobdii] [-V,--vo vo] [-c --count count]\n ",
				"\t[-o --offset offset] [-h,--help] [--version] file [...]\n");
		free (req);
		exit (errflg ? 1 : 0);
	}

	gfal_set_verbose (verbose);

	if (timeout > 0) {
		// for backward compatibility...
		gfal_set_timeout_connect (timeout_tmp);
		gfal_set_timeout_sendreceive (timeout_tmp);
		gfal_set_timeout_bdii (timeout_tmp);
		gfal_set_timeout_srm (timeout_tmp);
	}

	if (check_surls (argc - optind, argv + optind, NULL, 0) < 0) {
		perror ("lcg-ls");
		exit (1);
	}

	gfal_set_nobdii (req->no_bdii_check);
	if (vo && gfal_set_vo (vo) < 0) {
		perror ("lcg-ls");
		exit (1);
	}

	if (!strncmp (argv[optind], "lfn:", 4)) {
		free (req);

        if (count || offset) 
        {
			fprintf (stderr, "lcg-ls: WARNING: Count/offset options are ignored with LFN-s\n");
        }

		errflg = display_lfns_results ((const char **)(argv + optind), argc - optind, longlisting, dir ? 0 : 1);
	} else if (!strncmp (argv[optind], "srm:", 4) || !strncmp (argv[optind], "sfn:", 4)) {
		req->surls = argv + optind;
		req->nbfiles = argc - optind;
        req->srmv2_lscount = count;
        req->srmv2_lsoffset = offset;

		if (!dir)
			req->srmv2_lslevels = 1;
		else
			req->srmv2_lslevels = 0;

		if (gfal_init (req, &internal, NULL, 0) < 0) {
			free (req);
			exit (1);
		}
		free (req);

		if (verbose)
			printf ("SE type: %s\n", lcg_se_type_string (internal->setype));

		if (dir && internal->setype == TYPE_SE) {
			gfal_internal_free (internal);
			fprintf (stderr, "lcg-ls: Option '-d' is not compatible with Classic SEs\n");
			exit (1);
		}

		do {
			if (gfal_ls (internal, NULL, 0) < 0) {
				gfal_internal_free (internal);
				exit (1);
			}

			if ((n = gfal_get_results (internal, &filestatuses)) < 0) {
				gfal_internal_free (internal);
				fprintf (stderr, "lcg-ls: Cannot get results !\n");
				exit (1);
			}

			errflg += display_surls_results (filestatuses, n, longlisting, dir ? 0 : 1, 1);
		} while (!gfal_ls_end (internal, NULL, 0));

		gfal_internal_free (internal);
	} else {
			fprintf (stderr, "lcg-ls: files must be lfn:, srm: or sfn:\n");
			exit (1);
	}

	exit (errflg ? 1 : 0);
}


	int
display_lfns_results (const char **lfns, int size, int longlisting, int levels)
{
	int i, j, nbdir = 0;
	int dirs[NBDIR_MAX];
	struct stat64 statbuf;
	struct dirent *dirent;
	DIR *dir;
	char fname[GFAL_PATH_MAXLEN];
	char dirname[GFAL_PATH_MAXLEN];
	int err = 0;

	for (i = 0; i < size; ++i) {
		if (canonical_url (lfns[i], NULL, fname, GFAL_PATH_MAXLEN, NULL, 0) < 0) {
			perror (lfns[i]);
			--err;
			continue;
		}

		if (gfal_stat64 (fname, &statbuf) < 0) {
			perror (lfns[i]);
			--err;
			continue;
		}

		if (S_ISDIR (statbuf.st_mode)) {
			dirs[nbdir++] = i;
			if (levels > 0) continue;
		}

		if (!longlisting) {
			printf ("%s\n", fname);
		} else {
			char type;
			char sizebuf[21];

			if (S_ISDIR (statbuf.st_mode))
				type = 'd';
			else if (S_ISLNK (statbuf.st_mode))
				type = 'l';
			else type = '-';

			sprintf (sizebuf, GFAL_LONG64_FORMAT, (GFAL_LONG64) statbuf.st_size);
			printf ("%c%c%c%c%c%c%c%c%c%c %3d %5d %5d %11s %s\n", \
					type, \
					statbuf.st_mode & S_IRUSR ? 'r' : '-', \
					statbuf.st_mode & S_IWUSR ? 'w' : '-', \
					statbuf.st_mode & S_IXUSR ? 'x' : '-', \
					statbuf.st_mode & S_IRGRP ? 'r' : '-', \
					statbuf.st_mode & S_IWGRP ? 'w' : '-', \
					statbuf.st_mode & S_IXGRP ? 'x' : '-', \
					statbuf.st_mode & S_IROTH ? 'r' : '-', \
					statbuf.st_mode & S_IWOTH ? 'w' : '-', \
					statbuf.st_mode & S_IXOTH ? 'x' : '-', \
					(int) statbuf.st_nlink, \
					(int) statbuf.st_uid, \
					(int) statbuf.st_gid, \
					sizebuf, \
					fname);
		}
	}

	if (levels > 0 && nbdir > 0) {
		for (i = 0; i < nbdir; ++i) {
			j = dirs[i];

			if (canonical_url (lfns[j], NULL, dirname, GFAL_PATH_MAXLEN, NULL, 0) < 0) {
				perror (lfns[i]);
				--err;
				continue;
			}

			if (size > 1)
				printf ("\n%s:\n", dirname);

			if ((dir = gfal_opendir (dirname)) == NULL) {
				fprintf (stderr, "%s: %s\n", dirname, strerror (errno));
				--err;
				continue;
			}

			while ((dirent = gfal_readdir (dir)) != NULL) {
				if (strlen (dirname + 4) + strlen (dirent->d_name) + 1 >= GFAL_PATH_MAXLEN) {
					fprintf (stderr, "%s/%s: file name too long\n", dirname, dirent->d_name);
					--err;
					continue;
				}
				snprintf (fname, GFAL_PATH_MAXLEN, "%s/%s", dirname, dirent->d_name);

				if (gfal_stat64 (fname, &statbuf) < 0) {
					perror (fname);
					--err;
					continue;
				}

				if (!longlisting) {
					printf ("%s\n", dirent->d_name);
				} else {
					char type;
					char sizebuf[21];

					if (S_ISDIR (statbuf.st_mode))
						type = 'd';
					else if (S_ISLNK (statbuf.st_mode))
						type = 'l';
					else type = '-';

					sprintf (sizebuf, GFAL_LONG64_FORMAT, (GFAL_LONG64) statbuf.st_size);
					printf ("%c%c%c%c%c%c%c%c%c%c %3d %5d %5d %11s %s\n", \
							type, \
							statbuf.st_mode & S_IRUSR ? 'r' : '-', \
							statbuf.st_mode & S_IWUSR ? 'w' : '-', \
							statbuf.st_mode & S_IXUSR ? 'x' : '-', \
							statbuf.st_mode & S_IRGRP ? 'r' : '-', \
							statbuf.st_mode & S_IWGRP ? 'w' : '-', \
							statbuf.st_mode & S_IXGRP ? 'x' : '-', \
							statbuf.st_mode & S_IROTH ? 'r' : '-', \
							statbuf.st_mode & S_IWOTH ? 'w' : '-', \
							statbuf.st_mode & S_IXOTH ? 'x' : '-', \
							(int) statbuf.st_nlink, \
							(int) statbuf.st_uid, \
							(int) statbuf.st_gid, \
							sizebuf, \
							dirent->d_name);
				}
			}

			gfal_closedir (dir);
		}
	}

	return (err < 0 ? -1 : 0);
}

	int
display_surls_results (gfal_filestatus *filestatuses, int size, int longlisting, int levels, int first)
{
	int i, j, nbdir = 0;
	int dirs[NBDIR_MAX];
	int err = 0;

	if (first && levels > 0 && size == 1 && filestatuses[0].subpaths != NULL)
		return (display_surls_results (filestatuses[0].subpaths, filestatuses[0].nbsubpaths, longlisting, levels - 1, 0));

	for (i = 0; i < size; ++i) {
		if (!filestatuses[i].surl)
			/* skip the entry if there is no surl */
			continue;
		if (!filestatuses[i].status) {
			if (filestatuses[i].subpaths != NULL)
				dirs[nbdir++] = i;

			if (!longlisting) {
				printf ("%s\n", filestatuses[i].surl);
			} else {
				char type;
				char sizebuf[21];

				if (S_ISDIR (filestatuses[i].stat.st_mode))
					type = 'd';
				else if (S_ISLNK (filestatuses[i].stat.st_mode))
					type = 'l';
				else type = '-';

				sprintf (sizebuf, GFAL_LONG64_FORMAT, filestatuses[i].stat.st_size);
				printf ("%c%c%c%c%c%c%c%c%c%c %3d %5d %5d %7s %20s %s\n", \
						type, \
						filestatuses[i].stat.st_mode & S_IRUSR ? 'r' : '-', \
						filestatuses[i].stat.st_mode & S_IWUSR ? 'w' : '-', \
						filestatuses[i].stat.st_mode & S_IXUSR ? 'x' : '-', \
						filestatuses[i].stat.st_mode & S_IRGRP ? 'r' : '-', \
						filestatuses[i].stat.st_mode & S_IWGRP ? 'w' : '-', \
						filestatuses[i].stat.st_mode & S_IXGRP ? 'x' : '-', \
						filestatuses[i].stat.st_mode & S_IROTH ? 'r' : '-', \
						filestatuses[i].stat.st_mode & S_IWOTH ? 'w' : '-', \
						filestatuses[i].stat.st_mode & S_IXOTH ? 'x' : '-', \
						(int) filestatuses[i].stat.st_nlink, \
						(int) filestatuses[i].stat.st_uid, \
						(int) filestatuses[i].stat.st_gid, \
						sizebuf, \
						locality_desc[filestatuses[i].locality], \
						filestatuses[i].surl);

				if (filestatuses[i].checksumtype && filestatuses[i].checksum)
					printf ("\t* Checksum: %s (%s)\n", filestatuses[i].checksum, filestatuses[i].checksumtype);

				if (filestatuses[i].nbspacetokens > 0 && filestatuses[i].spacetokens != NULL) {
					printf ("\t* Space tokens:");

					for (j = 0; j < filestatuses[i].nbspacetokens; ++j)
						printf (" %s", filestatuses[i].spacetokens[j]);

					printf ("\n");
				}
			}
		} else {
			--err;
			if (filestatuses[i].explanation)
				fprintf (stderr, "%s: %s\n", filestatuses[i].surl, filestatuses[i].explanation);
			else
				fprintf (stderr, "%s: %s\n", filestatuses[i].surl, strerror (filestatuses[i].status));
		}
	}

	if (levels > 0 && nbdir > 0) {
		for (i = 0; i < nbdir; ++i) {
			j = dirs[i];
			printf ("\n%s:\n", filestatuses[j].surl);
			err += display_surls_results (filestatuses[j].subpaths, filestatuses[j].nbsubpaths, longlisting, levels - 1, 0);
		}
	}

	return (err < 0 ? -1 : 0);
}
