/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef lint
static char sccsid[] = "@(#)$RCSfile: lcg-stmd.c,v $ $Revision: 1.6 $ $Date: 2008/11/21 15:19:48 $ CERN Remi Mollon";
#endif /* not lint */

/* lcg_stmd - get space tokens associated to a space token description, and their metadata */
#include <sys/types.h>
#include <errno.h>
#include <getopt.h>
#include <stdlib.h>
#include <stdio.h>
#include "lcg_util.h"

static const char *policy_desc[] = {
	"UNKNOWN",
	"REPLICA",
	"OUTPUT",
	"CUSTODIAL"
};

static const char *latency_desc[] = {
	"UNKNOWN",
	"ONLINE",
	"NEARLINE"
};

	int
main (int argc, char **argv)
{
	int helpflg = 0;
	int insecure = 0;
	int verbose = 0;
	int version = 0;
	int i, c = 0;
	int errflg = 0;
	int timeout = 0, timeout_tmp = 0;
	static struct option longopts[] = {
		{"help", no_argument, NULL, 'h'},
		{"insecure", no_argument, NULL, 'i'},
		{"st", required_argument, NULL, 's'},
		{"nobdii", no_argument, NULL, 'b'},
		{"endpoint", required_argument, NULL, 'e'},
		{"timeout", required_argument, NULL, 't'},
		{"connect-timeout", required_argument, NULL, 130},
		{"sendreceive-timeout", required_argument, NULL, 131},
		{"bdii-timeout", required_argument, NULL, 132},
		{"srm-timeout", required_argument, NULL, 133},
		{"verbose", no_argument, NULL, 'v'},
		{"version", no_argument, NULL, 'W'},
		{0, 0, 0, 0}
	};
	char *spacetokendesc = NULL;
	char *endpoint = NULL;
	char *dp;
	int nobdii = 0;
	int nbtokens = 0;
	gfal_spacemd *spacemd = NULL;
	char sizebuf[21];

	while ((c = getopt_long (argc, argv, "hivt:s:e:bW", longopts, NULL)) != EOF) {
		switch (c) {
			case 'h':
				helpflg = 1;
				break;
			case 'i':
				insecure = 1;
				break;
			case 't':
				fprintf (stderr, "Warning: -t,--timeout is deprecated! Use --timeout-* options instead\n");
				if ((timeout = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--timeout': %s\n", optarg);
					errflg++;
				}
			case 130: // "connect-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--connect-timeout': %s\n", optarg);
					errflg++;
				} else
					gfal_set_timeout_connect (timeout_tmp);
				break;
			case 131: // "sendreceive-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--sendreceive-timeout': %s\n", optarg);
					errflg++;
				} else
					gfal_set_timeout_sendreceive (timeout_tmp);
				break;
			case 132: // "bdii-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--bdii-timeout': %s\n", optarg);
					errflg++;
				} else
					gfal_set_timeout_bdii (timeout_tmp);
				break;
			case 133: // "srm-timeout"
				if ((timeout_tmp = strtol (optarg, &dp, 10)) < 0 || 
						*dp != '\0' ) {
					fprintf (stderr,
							"invalid value for '--srm-timeout': %s\n", optarg);
					errflg++;
				} else
					gfal_set_timeout_srm (timeout_tmp);
				break;
			case 's':
				spacetokendesc = optarg;
				break;
			case 'e':
				endpoint = optarg;
				break;
			case 'b':
				nobdii = 1;
				break;
			case 'v':
				++verbose;
				break;
			case 'W':
				version = 1;
				break;
			case '?':
			default:
				errflg++;
				break;
		}
	}
	if (version) {
		printf ("lcg_util-%s\n", lcg_util_version ());
		printf ("GFAL-client-%s\n", gfal_version ());
		exit (0);
	}
	if (optind != argc || argc < 5)
		errflg++;
	if (errflg || helpflg) {
		fprintf (stderr, "usage: %s %s%s%s", argv[0],
				"[--connect-timeout value] [--sendreceive-timeout value] [--bdii-timeout value]\n",
				"\t[--srm-timeout value] [-h,--help] [-b,--nobdii] [-v,--verbose]\n",    
				"\t[--version] -e,--endpoint endpoint -s,--st spacetokendesc\n");
		exit (errflg ? 1 : 0);
	}

	if (!endpoint || !spacetokendesc) {
		fprintf (stderr, "lcg-stmd: you must specify at least 'endpoint' and 'spacetokendesc'\n");
		exit (1);
	}

	if (lcg_stmd (spacetokendesc, endpoint, &nbtokens, &spacemd, nobdii, timeout, NULL, 0) < 0) {
		if (errno)
			perror ("lcg_stmd");
		exit (1);
	}

	for (i = 0; i < nbtokens; ++i) {
		printf ("+ Space Token: %s\n", spacemd[i].spacetoken);
		printf ("  - Owner: %s\n", spacemd[i].owner);
		snprintf (sizebuf, 21, GFAL_LONG64_FORMAT, spacemd[i].totalsize);
		printf ("  - Total Size: %s\n", sizebuf);
		snprintf (sizebuf, 21, GFAL_LONG64_FORMAT, spacemd[i].guaranteedsize);
		printf ("  - Guaranteed Size: %s\n", sizebuf);
		snprintf (sizebuf, 21, GFAL_LONG64_FORMAT, spacemd[i].unusedsize);
		printf ("  - Unused Size: %s\n", sizebuf);
		printf ("  - Assigned Lifetime: %d\n", spacemd[i].lifetimeassigned);
		printf ("  - Left Lifetime: %d\n", spacemd[i].lifetimeleft);
		printf ("  - Retention Policy: %s\n", policy_desc[spacemd[i].retentionpolicy]);
		printf ("  - Access Latency: %s\n", latency_desc[spacemd[i].accesslatency]);
	}

	gfal_spacemd_free (nbtokens, spacemd);
	exit (0);
}
